import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*;
import java.util.*;

public class ListaAlumnos extends HttpServlet {

  Connection conn = null;
  // Vector que contendr los objetos Alumno
  Vector vectorAlumnos=null;

  // Este mtodo se ejecuta una nica vez (al ser inicializado por primera vez el servlet)
  // Se suelen inicializar variables y ejecutar operaciones costosas en tiempo 
  // de ejecucin (abrir ficheros, conectar con bases de datos, etc) 
  public void init (ServletConfig config) throws ServletException {

    // Llamada al mtodo init() de la superclase (GenericServlet)
    // As se asegura una correcta inicializacin del servlet
    super.init(config);

    // url de la base de datos
    String url=new String("jdbc:odbc:alumnos");

    // Carga del Driver
    try {
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch(ClassNotFoundException ex) {
      System.out.println("Error al cargar el driver");
      System.out.println(ex.getMessage());
    }

    // Establecimiento de la conexin
    try {
      conn=DriverManager.getConnection(url,"","");
    } catch (SQLException sqlEx) {
      System.out.println("Se ha producido un error al establecer la conexin con: "+url);
      System.out.println(sqlEx.getMessage());
    }

    System.out.println("Iniciando ListaAlumnos");
  } // fin del mtodo init()

  // Este mtodo es llamado por el servidor web al "apagarse" (al hacer shut down). Sirve para 
  // proporcionar una correcta desconexin de una base de datos, cerrar ficheros abiertos, etc.
  public void destroy () {
    super.destroy();
    System.out.println("Cerrando conexion...");
    try {
      conn.close();
    } catch(SQLException ex){
      System.out.println("No se pudo cerrar la conexion");
      System.out.println(ex.getMessage());
    }
  } // fin de destroy()

  // Mtodo llamada mediante un HTTP POST    
  public void doGet (HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {

    // Obtencin del grupo de prcticas
    String grupo = null;
    grupo = req.getParameter("GRUPO");

    if(grupo==null) {
      resp.sendError(500, "Se ha producido un error en la lectura de la solicitud"); 
      return;
    }

    //Consulta a la base de datos para obtener la lista de alumnos de un grupo
    if(obtenerLista(grupo)==0) {
      // Mostrar la lista de alumnos mediante una pgina HTML
      mostrarListaAlumnos(resp, grupo);
    }
    else if(obtenerLista(grupo)==-3) {
      resp.sendError(500, "No se ha encontrado el grupo: " +grupo);
    }
    else
      resp.sendError(500, "Se ha producido un error en el acceso a la base de datos");
  } // fin del mtodo doGet()

  public int obtenerLista(String grupo) {

    Statement stmt = null;
    ResultSet rs = null;

    // Ejecucin del query
    try {
      stmt=conn.createStatement();
      rs = stmt.executeQuery("SELECT DISTINCT Nombre, Apellidos, Curso, " +
           "Carnet, GrupoPractica FROM TablaAlumnos WHERE GrupoPractica = " + grupo + " ORDER BY Carnet");

      vectorAlumnos=new Vector();

      // Lectura del ResultSet, en Java 2
      while (rs.next()) {
        Alumno temp=new Alumno();
        temp.setNombre(rs.getString("Nombre"));
        temp.setApellidos(rs.getString("Apellidos"));
        temp.setCarnet(rs.getLong("Carnet"));
        temp.setCurso(rs.getInt("Curso"));
        vectorAlumnos.addElement(temp);
      }
      // En el JDK 1.1.x hay un bug que hace que no se lean bien los valores 
      // que no son Strings. Sera de la siguiente forma:
      /* while (rs.next()) {
           Alumno temp=new Alumno();
           temp.setNombre(rs.getString("Nombre"));
           temp.setApellidos(rs.getString("Apellidos"));
           temp.setCarnet(java.lang.Long.parseLong(rs.getString("Carnet")));
           temp.setCurso(java.lang.Integer.parseInt(rs.getString("Curso")));
           vectorAlumnos.addElement(temp);
         }*/

      if(vectorAlumnos.size()==0) return -3;

      return 0;

    } catch (SQLException sql) {
      System.out.println("Se produjo un error al crear el Statement");
      System.out.println(sql.getMessage());
      return -1;
    } finally {
      // se cierra el Statment
      if(stmt!=null) {
        try { 
          stmt.close();
        } catch(SQLException e) {
          System.out.println("Error al cerrar el Statement");
          System.out.println(e.getMessage());
          return -2;
        }
      }
    } // fin del finally
  } // fin del mtodo obtenerLista()

  public void mostrarListaAlumnos(HttpServletResponse resp, String grupo) {

    // se establece el tipo de contenido MIME de la respuesta
    resp.setContentType("text/html");

    // se obtiene un PrintWriter donde escribir (slo para mandar texto)
    PrintWriter out=null;
    try {
      out=resp.getWriter();
    } catch (IOException io) {
      System.out.println("Se ha producido una excepcion");
    }

    // se manda la lista
    out.println("<html>");
    out.println("");
    out.println("<head>");
    out.println("<title>Lista de alumnos del grupo "+grupo+"</title>");
    out.println("<meta name=\"GENERATOR\" content=\"Microsoft FrontPage 3.0\">");
    out.println("</head>");
    out.println("");
    out.println("<body>");
    out.println("");
    out.println("<H2 align=\"center\">Lista de alumnos del grupo " + grupo + "</H2>");
    out.println("<div align=\"center\"><center>");
    out.println("");
    out.println("<table border=\"1\" width=\"70%\">");
    out.println("<tr>");
    out.println("<td width=\"25%\" bgcolor=\"#808080\"><font color=\"#FFFFFF\">Carnet</font></td>");
    out.println("<td width=\"25%\" bgcolor=\"#808080\"><font color=\"#FFFFFF\">Nombre</font></td>");
    out.println("<td width=\"25%\" bgcolor=\"#808080\"><font color=\"#FFFFFF\">Apellidos</font></td>");
    out.println("<td width=\"25%\" bgcolor=\"#808080\"><font color=\"#FFFFFF\">Curso</font></td>");
    out.println("</tr>");

    // Datos del Alumno por filas
    Alumno alum=null;
    for (int i=0; i<vectorAlumnos.size();i++) {
      alum=(Alumno)vectorAlumnos.elementAt(i);
      out.println("<tr>");
      out.println("<td width=\"25%\">"+alum.getCarnet()+"</td>");
      out.println("<td width=\"25%\">"+alum.getNombre()+"</td>");
      out.println("<td width=\"25%\">"+alum.getApellidos()+"</td>");
      out.println("<td width=\"25%\">"+alum.getCurso()+"</td>");
      out.println("</tr>");
    }

    out.println("</table>");
    out.println("</center></div>");
    out.println("</body>");
    out.println("</html>");

    // se fuerza la descarga del buffer y se cierra el PrintWriter
    out.flush();
    out.close();
  } // fin del mtodo mostrarListaAlumnos()

  // Funcin que permite al servidor web obtener una pequea descripcin del
  // servlet, qu cometido tiene, nombre del autor, comentarios adicionales...
  public String getServletInfo() {
    return "Servlet que muestra en pantalla la lista de un grupo de prcticas";
  }
} // fin de la clase ListaAlumnos
