import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*;

public class ServletOpinion2 extends HttpServlet {

  // Declaracin de variables miembro
  private String nombre = null;
  private String apellidos = null;
  private String opinion = null;
  private String comentarios = null;

  // Referencia a un objeto de la interface java.sql.Connection 
  Connection conn = null;

  // Este mtodo se ejecuta una nica vez (al ser inicializado el servlet por primera vez)
  // Se suelen inicializar variables y ejecutar operaciones costosas en tiempo
  // de ejecucin (abrir ficheros, conectar con bases de datos, etc) 
  public void init (ServletConfig config) throws ServletException {

    // Llamada al mtodo init() de la superclase (GenericServlet)
    // As se asegura una correcta inicializacin del servlet
    super.init(config);

    // dsn (Data Source Name) de la base de datos
    String dsn = new String("jdbc:odbc:opinion");

    // Carga del Driver del puente JDBC-ODBC
    try {
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch(ClassNotFoundException ex) {
      System.out.println("Error al cargar el driver");
      System.out.println(ex.getMessage());
    }
    // Establecimiento de la conexin con la base de datos
    try {
      conn = DriverManager.getConnection(dsn, "", "");
    } catch (SQLException sqlEx) {
      System.out.println("Se ha producido un error al" +
                         " establecer la conexin con: " + dsn);
      System.out.println(sqlEx.getMessage());
    }

    System.out.println("Iniciando ServletOpinion (version BD)...");
  } // fin del mtodo init()

  // Este mtodo es llamado por el servidor web al "apagarse" (shut down).
  // Sirve para proporcionar una correcta desconexin de una base de datos, 
  // cerrar ficheros abiertos, etc.
  public void destroy () {
    super.destroy();
    System.out.println("Cerrando conexion...");
    try {
      conn.close();
    } catch(SQLException ex){
      System.out.println("No se pudo cerrar la conexion");
      System.out.println(ex.getMessage());
    }
  } // fin del mtodo destroy()

  // Mtodo de llamada mediante un HTTP POST 
  public void doPost (HttpServletRequest req, HttpServletResponse resp)
                                     throws ServletException, IOException {

    boolean hayError = false;

    // adquisicin de los valores del formulario
    if(req.getParameter("nombre")!="")
      nombre = req.getParameter("nombre");
    else 
      hayError=true;
            
    if(req.getParameter("apellidos")!="")
      apellidos = req.getParameter("apellidos");
    else
      hayError = true;

    if(req.getParameter("opinion")!=null)
      opinion = req.getParameter("opinion");
    else 
      hayError = true;

    if(req.getParameter("comentarios")!="")
      comentarios = req.getParameter("comentarios");
    else
      hayError = true;

    // Mandar al usuario los valores adquiridos (Si no se ha producido error)
    if(!hayError) {
      if (actualizarBaseDeDatos() == 0)
        devolverPaginaHTML(resp);
      else
        resp.sendError(500, "Se ha producido un error al actualizar la base de datos");
    } else
      resp.sendError(500, "Se ha producido un error en la adquisicin de parmetros");
  } // fin doPost()

  public int actualizarBaseDeDatos() {
    // crear un statement de SQL
    Statement stmt=null;
    int numeroFilasActualizadas=0;

    // Ejecucin del query de actualizacin de la base de datos
    try {
      stmt = conn.createStatement();
      numeroFilasActualizadas = stmt.executeUpdate("INSERT INTO Opiniones_Recogidas VALUES "+
                              "('"+nombre+"','"+apellidos+"','"+opinion+"','"+comentarios+"')");
      if(numeroFilasActualizadas!=1) return -1;
    } catch (SQLException sql) {
      System.out.println("Se produjo un error creando Statement");
      System.out.println(sql.getMessage());
      return -2;
    } finally {
      // Se cierra el Statement
      if(stmt!=null) {
        try { 
          stmt.close();
        } catch(SQLException e){
          System.out.println("Error cerrando Statement");
          System.out.println(e.getMessage());
          return -3;
        }
      }
      return 0;
    } // fin finally
  } // fin mtodo actualizarBaseDeDatos()

  public void devolverPaginaHTML(HttpServletResponse resp) {
    // Se obtiene un PrintWriter donde escribir (slo para mandar texto)
    PrintWriter out=null;
    try {
      out=resp.getWriter();
    } catch (IOException io) {
      System.out.println("Se ha producido una excepcion");    
    }

    // Se establece el tipo de contenido MIME de la respuesta
    resp.setContentType("text/html");

    // Se mandan los valores
    out.println("<html>");
    out.println("<head>");
    out.println("<title>Valores recogidos en el formulario</title>");
    out.println("</head>");
    out.println("<body>");
    out.println("<b><font size=+2>Valores recogidos del");
    out.println("formulario: </font></b>");
    out.println("<p><font size=+1><b>Nombre: </b>"+nombre+"</font>");
    out.println("<br><fontsize=+1><b>Apellido : </b>"+apellidos+"</font><b><font size=+1></font></b>");
    out.println("<p><font size=+1><b>Opini&oacute;n: "+"</b><i>"+opinion+"</i></font>");
    out.println("<br><font size=+1><b>Comentarios: </b>"+comentarios+"</font>");
    out.println("<P><HR><CENTER><H2>Valores actualizados con xito</CENTER>");
    out.println("</body>");
    out.println("</html>");

    // Se fuerza la descarga del buffer y se cierra el PrintWriter
    out.flush();
    out.close();
  } // fin de devolverPaginaHTML()

  // Funcin que permite al servidor web obtener una pequea descripcin del
  // servlet, qu cometido tiene, nombre del autor, comentarios adicionales...
  public String getServletInfo() {
    return "Este servlet lee los datos de un formulario y los introduce en una base da datos";
  } // fin de getServletInfo()

} // fin de la clase servletOpinion2
